/*
 *  jDTAUS - DTAUS fileformat.
 *  Copyright (c) 2005 Christian Schulte <cs@schulte.it>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License along
 *  with this program; if not, write to the Free Software Foundation, Inc.,
 *  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 */
package org.jdtaus.banking.dtaus.swing;

import java.awt.Color;
import java.awt.Dimension;
import java.awt.Frame;
import java.awt.Toolkit;
import java.text.ParseException;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Comparator;
import java.util.Currency;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Locale;
import java.util.Map;
import javax.swing.JDialog;
import javax.swing.JLabel;
import javax.swing.SwingUtilities;
import javax.swing.UIManager;
import org.jdtaus.banking.AlphaNumericText27;
import org.jdtaus.banking.Bankleitzahl;
import org.jdtaus.banking.BankleitzahlenVerzeichnis;
import org.jdtaus.banking.CurrencyDirectory;
import org.jdtaus.banking.Kontonummer;
import org.jdtaus.banking.Referenznummer10;
import org.jdtaus.banking.dtaus.Header;
import org.jdtaus.banking.dtaus.LogicalFileType;
import org.jdtaus.banking.swing.AlphaNumericText27TextField;
import org.jdtaus.banking.swing.BankleitzahlTextField;
import org.jdtaus.banking.swing.CurrencyRenderer;
import org.jdtaus.banking.swing.KontonummerTextField;
import org.jdtaus.banking.swing.Referenznummer10TextField;
import org.jdtaus.core.container.ContainerFactory;
import org.jdtaus.core.container.ContainerInitializer;
import org.jdtaus.core.container.ContextFactory;
import org.jdtaus.core.container.ContextInitializer;
import org.jdtaus.core.container.Implementation;
import org.jdtaus.core.container.ModelFactory;
import org.jdtaus.core.container.Properties;
import org.jdtaus.core.container.Property;

/**
 * Dialog to display and edit a {@code Header}.
 *
 * @author <a href="mailto:cs@schulte.it">Christian Schulte</a>
 * @version $Id: HeaderDialog.java 3324 2007-05-02 01:07:32Z schulte2005 $
 */
public class HeaderDialog extends JDialog implements ContainerInitializer
{

    //--Implementation----------------------------------------------------------

    // This section is managed by jdtaus-container-mojo.

    /** Meta-data describing the implementation. */
    private static final Implementation META =
        ModelFactory.getModel().getModules().
        getImplementation(HeaderDialog.class.getName());

    //----------------------------------------------------------Implementation--
    //--Constructors------------------------------------------------------------

    // This section is managed by jdtaus-container-mojo.

    /**
     * Initializes the properties of the instance.
     *
     * @param meta the property values to initialize the instance with.
     *
     * @throws NullPointerException if {@code meta} is {@code null}.
     */
    protected void initializeProperties(final Properties meta)
    {
        Property p;

        if(meta == null)
        {
            throw new NullPointerException("meta");
        }

    }

    //------------------------------------------------------------Constructors--
    //--ContainerInitializer----------------------------------------------------

    /** Initializes the instance centering the dialog on the screen. */
    public void initialize()
    {
        final Dimension screenSize = Toolkit.getDefaultToolkit().
            getScreenSize();

        this.initComponents();

        this.setLocation(screenSize.width / 2 - this.getWidth() / 2,
            screenSize.height / 2 - this.getHeight() / 2);

        this.updateCurrencies();
        this.updateFileTypes();
    }

    //----------------------------------------------------ContainerInitializer--
    //--Dependencies------------------------------------------------------------

    // This section is managed by jdtaus-container-mojo.

    /** Configured <code>BankleitzahlenVerzeichnis</code> implementation. */
    private transient BankleitzahlenVerzeichnis _dependency3;

    /**
     * Gets the configured <code>BankleitzahlenVerzeichnis</code> implementation.
     *
     * @return the configured <code>BankleitzahlenVerzeichnis</code> implementation.
     */
    private BankleitzahlenVerzeichnis getBankleitzahlenVerzeichnis()
    {
        BankleitzahlenVerzeichnis ret = null;
        if(this._dependency3 != null)
        {
            ret = this._dependency3;
        }
        else
        {
            ret = (BankleitzahlenVerzeichnis) ContainerFactory.getContainer().
                getDependency(HeaderDialog.class,
                "BankleitzahlenVerzeichnis");

            if(ModelFactory.getModel().getModules().
                getImplementation(HeaderDialog.class.getName()).
                getDependencies().getDependency("BankleitzahlenVerzeichnis").
                isBound())
            {
                this._dependency3 = ret;
            }
        }

        if(ret instanceof ContextInitializer && !((ContextInitializer) ret).
            isInitialized(ContextFactory.getContext()))
        {
            ((ContextInitializer) ret).initialize(ContextFactory.getContext());
        }

        return ret;
    }
    /** Configured <code>UserInterface</code> implementation. */
    private transient EditorGUI _dependency2;

    /**
     * Gets the configured <code>UserInterface</code> implementation.
     *
     * @return the configured <code>UserInterface</code> implementation.
     */
    private EditorGUI getUserInterface()
    {
        EditorGUI ret = null;
        if(this._dependency2 != null)
        {
            ret = this._dependency2;
        }
        else
        {
            ret = (EditorGUI) ContainerFactory.getContainer().
                getDependency(HeaderDialog.class,
                "UserInterface");

            if(ModelFactory.getModel().getModules().
                getImplementation(HeaderDialog.class.getName()).
                getDependencies().getDependency("UserInterface").
                isBound())
            {
                this._dependency2 = ret;
            }
        }

        if(ret instanceof ContextInitializer && !((ContextInitializer) ret).
            isInitialized(ContextFactory.getContext()))
        {
            ((ContextInitializer) ret).initialize(ContextFactory.getContext());
        }

        return ret;
    }
    /** Configured <code>Application</code> implementation. */
    private transient Editor _dependency1;

    /**
     * Gets the configured <code>Application</code> implementation.
     *
     * @return the configured <code>Application</code> implementation.
     */
    private Editor getApplication()
    {
        Editor ret = null;
        if(this._dependency1 != null)
        {
            ret = this._dependency1;
        }
        else
        {
            ret = (Editor) ContainerFactory.getContainer().
                getDependency(HeaderDialog.class,
                "Application");

            if(ModelFactory.getModel().getModules().
                getImplementation(HeaderDialog.class.getName()).
                getDependencies().getDependency("Application").
                isBound())
            {
                this._dependency1 = ret;
            }
        }

        if(ret instanceof ContextInitializer && !((ContextInitializer) ret).
            isInitialized(ContextFactory.getContext()))
        {
            ((ContextInitializer) ret).initialize(ContextFactory.getContext());
        }

        return ret;
    }
    /** Configured <code>CurrencyDirectory</code> implementation. */
    private transient CurrencyDirectory _dependency0;

    /**
     * Gets the configured <code>CurrencyDirectory</code> implementation.
     *
     * @return the configured <code>CurrencyDirectory</code> implementation.
     */
    private CurrencyDirectory getCurrencyDirectory()
    {
        CurrencyDirectory ret = null;
        if(this._dependency0 != null)
        {
            ret = this._dependency0;
        }
        else
        {
            ret = (CurrencyDirectory) ContainerFactory.getContainer().
                getDependency(HeaderDialog.class,
                "CurrencyDirectory");

            if(ModelFactory.getModel().getModules().
                getImplementation(HeaderDialog.class.getName()).
                getDependencies().getDependency("CurrencyDirectory").
                isBound())
            {
                this._dependency0 = ret;
            }
        }

        if(ret instanceof ContextInitializer && !((ContextInitializer) ret).
            isInitialized(ContextFactory.getContext()))
        {
            ((ContextInitializer) ret).initialize(ContextFactory.getContext());
        }

        return ret;
    }

    //------------------------------------------------------------Dependencies--
    //--Properties--------------------------------------------------------------

    // This section is managed by jdtaus-container-mojo.


    //--------------------------------------------------------------Properties--
    //--HeaderDialog------------------------------------------------------------

    /** A return status code - returned if Cancel button has been pressed */
    public static final int RET_CANCEL = 0;

    /** A return status code - returned if OK button has been pressed */
    public static final int RET_OK = 1;

    /**
     * Creates new {@code HeaderDialog} instance.
     *
     * @param owner the frame owning the dialog.
     * @param modal {@code true} for a modal dialog; {@code false} else.
     */
    public HeaderDialog(final Frame owner, final boolean modal)
    {
        super(owner, modal);
        this.initializeProperties(HeaderDialog.META.getProperties());
        this.initialize();
    }

    /**
     * Gets the return status of the dialog.
     *
     * @return the return status of this dialog - one of {@code RET_OK} or
     * {@code RET_CANCEL}.
     */
    public int getReturnStatus()
    {
        return this.returnStatus;
    }

    /**
     * Gets the A record displayed by the dialog.
     *
     * @return the A record displayed by the dialog or {@code null} if
     * the dialog displays invalid data.
     */
    public Header getHeader()
    {
        String str;

        boolean valid = true;
        final Header h = new Header();
        final Date createDate;
        final Date executionDate;
        final Preferences prefs = this.getApplication().getPreferences();

        try
        {
            str = this.customerTextField.getText();
            h.setCustomer(AlphaNumericText27.parse(str));
            this.resetForeground(this.customerLabel);
        }
        catch(ParseException e)
        {
            this.setForeground(this.customerLabel, Color.RED);
            this.customerTextField.setText(AlphaNumericText27.
                normalize(this.customerTextField.getText()));

            valid = false;
        }

        try
        {
            str = this.accountTextField.getText();
            h.setAccount(Kontonummer.parse(str));
            this.resetForeground(this.accountLabel);
        }
        catch(ParseException e)
        {
            this.setForeground(this.accountLabel, Color.RED);
            valid = false;
        }

        try
        {
            str = this.bankTextField.getText();
            h.setBank(Bankleitzahl.parse(str));

            if(prefs.isBankcodeValidationDisabled() ||
                isBankcodeValid(h.getBank()))
            {
                this.resetForeground(this.bankLabel);
            }
            else
            {
                this.setForeground(this.bankLabel, Color.RED);
                valid = false;
            }
        }
        catch(ParseException e)
        {
            this.setForeground(this.bankLabel, Color.RED);
            valid = false;
        }

        try
        {
            str = this.referenceTextField.getText();
            h.setReference(Referenznummer10.parse(str));
            this.resetForeground(this.referenceLabel);
        }
        catch(ParseException e)
        {
            this.setForeground(this.referenceLabel, Color.RED);
            valid = false;
        }

        createDate = (Date) this.createDateTextField.getValue();
        if(createDate == null || !Header.Schedule.checkDate(createDate))
        {
            this.setForeground(this.createDateLabel, Color.RED);
            valid = false;
        }
        else
        {
            this.resetForeground(this.createDateLabel);
        }

        executionDate = (Date) this.executionDateTextField.getValue();
        if(executionDate != null && !Header.Schedule.checkDate(executionDate))
        {
            this.setForeground(this.executionDateLabel, Color.RED);
            valid = false;
        }
        else
        {
            this.resetForeground(this.executionDateLabel);
        }

        if(createDate != null && !Header.Schedule.
            checkSchedule(createDate, executionDate))
        {
            this.setForeground(this.createDateLabel, Color.RED);
            this.setForeground(this.executionDateLabel, Color.RED);
            valid = false;
        }

        if(valid)
        {
            h.setSchedule(new Header.Schedule(createDate, executionDate));
        }

        h.setType((LogicalFileType) this.typeComboBox.getSelectedItem());
        if(h.getType() == null)
        {
            this.setForeground(this.typeLabel, Color.RED);
            valid = false;
        }
        else
        {
            this.resetForeground(this.typeLabel);
        }

        h.setCurrency((Currency) this.currencyComboBox.getSelectedItem());
        if(h.getCurrency() == null)
        {
            this.setForeground(this.currencyLabel, Color.RED);
            valid = false;
        }
        else
        {
            this.resetForeground(this.currencyLabel);
        }

        if(h.getType() != null)
        {
            if(h.getType().isSendByBank())
            {
                try
                {
                    str = this.bankDataTextField.getText();
                    h.setBankData(Bankleitzahl.parse(str));

                    if(prefs.isBankcodeValidationDisabled() ||
                        isBankcodeValid(h.getBankData()))
                    {
                        this.resetForeground(this.bankDataLabel);
                    }
                    else
                    {
                        this.setForeground(this.bankDataLabel, Color.RED);
                        valid = false;
                    }
                }
                catch(ParseException e)
                {
                    this.setForeground(this.bankDataLabel, Color.RED);
                    valid = false;
                }
            }
            else
            {
                h.setBankData(null);
            }
        }

        return valid ? h : null;
    }

    /**
     * Sets the A record to be displayed by the dialog.
     *
     * @param header the A record to be displayed by the dialog or
     * {@code null} to reset the dialog display.
     */
    public void setHeader(final Header header)
    {
        for(JLabel label : this.colors.keySet())
        {
            this.resetForeground(label);
        }

        this.date = header != null && header.getSchedule() != null ?
            header.getSchedule().getCreateDate() : null;

        this.updateCurrencies();

        if(header == null)
        {
            this.accountTextField.setText(null);
            this.bankTextField.setText(null);
            this.createDateTextField.setValue(null);
            this.currencyComboBox.setSelectedItem(null);
            this.customerTextField.setText(null);
            this.executionDateTextField.setValue(null);
            this.referenceTextField.setText(null);
            this.typeComboBox.setSelectedItem(null);
            this.bankDataTextField.setText(null);
        }
        else
        {
            this.accountTextField.setText(header.getAccount() == null ? null :
                header.getAccount().format(Kontonummer.ELECTRONIC_FORMAT));

            this.bankTextField.setText(header.getBank() == null ? null :
                header.getBank().format(Bankleitzahl.ELECTRONIC_FORMAT));

            this.currencyComboBox.setSelectedItem(header.getCurrency());

            this.customerTextField.setText(header.getCustomer() == null ? null :
                header.getCustomer().format().trim());

            this.referenceTextField.setText(header.getReference() == null ?
                null : header.getReference().
                format(Referenznummer10.ELECTRONIC_FORMAT));

            this.typeComboBox.setSelectedItem(header.getType());

            if(header.getSchedule() != null)
            {
                this.createDateTextField.setValue(this.date);
                this.executionDateTextField.setValue(header.getSchedule().
                    getExecutionDate());

            }
            else
            {
                this.createDateTextField.setValue(null);
                this.executionDateTextField.setValue(null);
            }

            this.bankDataTextField.setText(header.getBankData() == null ? null :
                header.getBankData().format(Bankleitzahl.ELECTRONIC_FORMAT));

        }
    }

    /**
     * Flag indicating if the dialog supports logical files holding debit
     * orders.
     *
     * @return {@code true} if the dialog supports logical files holding
     * debit orders; {@code false} if not.
     */
    public boolean isDebitSupported()
    {
        return this.debitSupported;
    }

    /**
     * Sets the flag indicating if the dialog supports logical files holding
     * debit orders.
     *
     * @param value {@code true} if the dialog should support logical files
     * holding debit orders; {@code false} if not.
     */
    public void setDebitSupported(final boolean value)
    {
        this.firePropertyChange("debitSupported", this.debitSupported, value);
        this.debitSupported = value;
        this.updateFileTypes();
    }

    /**
     * Flag indicating if the dialog supports logical files holding remittance
     * orders.
     *
     * @return {@code true} if the dialog supports logical files holding
     * remittance orders; {@code false} if not.
     */
    public boolean isRemittanceSupported()
    {
        return this.remittanceSupported;
    }

    /**
     * Sets the flag indicating if the dialog supports logical files holding
     * remittance orders.
     *
     * @param value {@code true} if the dialog should support logical files
     * holding remittance orders; {@code false} if not.
     */
    public void setRemittanceSupported(final boolean value)
    {
        this.firePropertyChange("remittanceSupported", this.remittanceSupported,
            value);

        this.remittanceSupported = value;
        this.updateFileTypes();
    }

    /** This method is called from within the constructor to
     * initialize the form.
     * WARNING: Do NOT modify this code. The content of this method is
     * always regenerated by the Form Editor.
     */
    // <editor-fold defaultstate="collapsed" desc=" Generated Code ">//GEN-BEGIN:initComponents
    private void initComponents()
    {
        okButton = new javax.swing.JButton();
        cancelButton = new javax.swing.JButton();
        headerPanel = new javax.swing.JPanel();
        typeLabel = new javax.swing.JLabel();
        typeComboBox = new javax.swing.JComboBox();
        customerLabel = new javax.swing.JLabel();
        customerTextField = new AlphaNumericText27TextField();
        accountLabel = new javax.swing.JLabel();
        accountTextField = new KontonummerTextField();
        bankLabel = new javax.swing.JLabel();
        bankTextField = new BankleitzahlTextField();
        currencyLabel = new javax.swing.JLabel();
        currencyComboBox = new javax.swing.JComboBox();
        referenceLabel = new javax.swing.JLabel();
        referenceTextField = new Referenznummer10TextField();
        createDateLabel = new javax.swing.JLabel();
        createDateTextField = new javax.swing.JFormattedTextField();
        executionDateLabel = new javax.swing.JLabel();
        executionDateTextField = new javax.swing.JFormattedTextField();
        searchBankleitzahlButton = new javax.swing.JButton();
        bankDataLabel = new javax.swing.JLabel();
        bankDataTextField = new BankleitzahlTextField();
        searchBankDataButton = new javax.swing.JButton();

        setDefaultCloseOperation(javax.swing.WindowConstants.DO_NOTHING_ON_CLOSE);
        setTitle(HeaderDialogBundle.getTitleText(Locale.getDefault()));
        setName("Header Dialog");
        addWindowListener(new java.awt.event.WindowAdapter()
        {
            public void windowClosing(java.awt.event.WindowEvent evt)
            {
                closeDialog(evt);
            }
        });

        okButton.setText("OK");
        okButton.addActionListener(new java.awt.event.ActionListener()
        {
            public void actionPerformed(java.awt.event.ActionEvent evt)
            {
                okButtonActionPerformed(evt);
            }
        });

        cancelButton.setText("Cancel");
        cancelButton.addActionListener(new java.awt.event.ActionListener()
        {
            public void actionPerformed(java.awt.event.ActionEvent evt)
            {
                cancelButtonActionPerformed(evt);
            }
        });

        headerPanel.setBorder(javax.swing.BorderFactory.createTitledBorder(HeaderDialogBundle.getHeaderPanelTitleText(Locale.getDefault())));
        typeLabel.setText(HeaderDialogBundle.getTypeLabelText(Locale.getDefault()));

        typeComboBox.setRenderer(new LogicalFileTypeRenderer());
        typeComboBox.addActionListener(new java.awt.event.ActionListener()
        {
            public void actionPerformed(java.awt.event.ActionEvent evt)
            {
                typeComboBoxActionPerformed(evt);
            }
        });

        customerLabel.setText(HeaderDialogBundle.getCustomerLabelText(Locale.getDefault()));

        customerTextField.setColumns(AlphaNumericText27.MAX_LENGTH);

        accountLabel.setText(HeaderDialogBundle.getAccountLabelText(Locale.getDefault()));

        accountTextField.setColumns(Kontonummer.MAX_CHARACTERS);

        bankLabel.setText(HeaderDialogBundle.getBankLabelText(Locale.getDefault()));

        bankTextField.setColumns(Bankleitzahl.MAX_CHARACTERS);

        currencyLabel.setText(HeaderDialogBundle.getCurrencyLabelText(Locale.getDefault()));

        currencyComboBox.setRenderer(new CurrencyRenderer());

        referenceLabel.setText(HeaderDialogBundle.getReferenceLabelText(Locale.getDefault()));

        referenceTextField.setColumns(Referenznummer10.MAX_CHARACTERS);

        createDateLabel.setText(HeaderDialogBundle.getCreateDateLabelText(Locale.getDefault()));

        createDateTextField.setValue(new Date());
        createDateTextField.addPropertyChangeListener(new java.beans.PropertyChangeListener()
        {
            public void propertyChange(java.beans.PropertyChangeEvent evt)
            {
                createDateTextFieldPropertyChange(evt);
            }
        });

        executionDateLabel.setText(HeaderDialogBundle.getExecutionDateLabelText(Locale.getDefault()));

        executionDateTextField.setValue(new Date());

        searchBankleitzahlButton.setAction(new SearchBankleitzahlAction(this.bankTextField, true));
        searchBankleitzahlButton.setBorderPainted(false);
        searchBankleitzahlButton.setContentAreaFilled(false);
        searchBankleitzahlButton.setDefaultCapable(false);
        searchBankleitzahlButton.setMaximumSize(new java.awt.Dimension(16, 16));
        searchBankleitzahlButton.setMinimumSize(new java.awt.Dimension(16, 16));
        searchBankleitzahlButton.setPreferredSize(new java.awt.Dimension(16, 16));

        bankDataLabel.setText(HeaderDialogBundle.getBankDataLabelText(Locale.getDefault()));

        bankDataTextField.setEnabled(false);

        searchBankDataButton.setAction(new SearchBankleitzahlAction(this.bankDataTextField, true));
        searchBankDataButton.setBorderPainted(false);
        searchBankDataButton.setContentAreaFilled(false);
        searchBankDataButton.setDefaultCapable(false);
        searchBankDataButton.setMaximumSize(new java.awt.Dimension(16, 16));
        searchBankDataButton.setMinimumSize(new java.awt.Dimension(16, 16));
        searchBankDataButton.setPreferredSize(new java.awt.Dimension(16, 16));

        org.jdesktop.layout.GroupLayout headerPanelLayout = new org.jdesktop.layout.GroupLayout(headerPanel);
        headerPanel.setLayout(headerPanelLayout);
        headerPanelLayout.setHorizontalGroup(
            headerPanelLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
            .add(headerPanelLayout.createSequentialGroup()
                .addContainerGap()
                .add(headerPanelLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.TRAILING, false)
                    .add(org.jdesktop.layout.GroupLayout.LEADING, bankDataLabel, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                    .add(org.jdesktop.layout.GroupLayout.LEADING, executionDateLabel, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                    .add(org.jdesktop.layout.GroupLayout.LEADING, createDateLabel, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                    .add(org.jdesktop.layout.GroupLayout.LEADING, referenceLabel, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                    .add(org.jdesktop.layout.GroupLayout.LEADING, currencyLabel, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                    .add(org.jdesktop.layout.GroupLayout.LEADING, bankLabel, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                    .add(org.jdesktop.layout.GroupLayout.LEADING, accountLabel, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                    .add(org.jdesktop.layout.GroupLayout.LEADING, customerLabel, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                    .add(org.jdesktop.layout.GroupLayout.LEADING, typeLabel, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, 135, Short.MAX_VALUE))
                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                .add(headerPanelLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
                    .add(createDateTextField, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, 319, Short.MAX_VALUE)
                    .add(referenceTextField, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, 319, Short.MAX_VALUE)
                    .add(currencyComboBox, 0, 319, Short.MAX_VALUE)
                    .add(org.jdesktop.layout.GroupLayout.TRAILING, headerPanelLayout.createSequentialGroup()
                        .add(bankTextField, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, 297, Short.MAX_VALUE)
                        .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                        .add(searchBankleitzahlButton, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE))
                    .add(accountTextField, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, 319, Short.MAX_VALUE)
                    .add(customerTextField, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, 319, Short.MAX_VALUE)
                    .add(typeComboBox, 0, 319, Short.MAX_VALUE)
                    .add(org.jdesktop.layout.GroupLayout.TRAILING, executionDateTextField, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, 319, Short.MAX_VALUE)
                    .add(org.jdesktop.layout.GroupLayout.TRAILING, headerPanelLayout.createSequentialGroup()
                        .add(bankDataTextField, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, 297, Short.MAX_VALUE)
                        .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                        .add(searchBankDataButton, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)))
                .addContainerGap())
        );
        headerPanelLayout.setVerticalGroup(
            headerPanelLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
            .add(headerPanelLayout.createSequentialGroup()
                .add(headerPanelLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.BASELINE)
                    .add(typeComboBox, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                    .add(typeLabel))
                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                .add(headerPanelLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.BASELINE)
                    .add(customerTextField, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                    .add(customerLabel))
                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                .add(headerPanelLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.BASELINE)
                    .add(accountTextField, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                    .add(accountLabel))
                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                .add(headerPanelLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.BASELINE)
                    .add(bankLabel)
                    .add(bankTextField, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                    .add(searchBankleitzahlButton, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE))
                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                .add(headerPanelLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.BASELINE)
                    .add(currencyComboBox, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                    .add(currencyLabel))
                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                .add(headerPanelLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.BASELINE)
                    .add(referenceTextField, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                    .add(referenceLabel))
                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                .add(headerPanelLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.BASELINE)
                    .add(createDateTextField, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                    .add(createDateLabel))
                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                .add(headerPanelLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.BASELINE)
                    .add(executionDateLabel)
                    .add(executionDateTextField, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE))
                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                .add(headerPanelLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.BASELINE)
                    .add(bankDataLabel)
                    .add(searchBankDataButton, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                    .add(bankDataTextField, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE))
                .add(38, 38, 38))
        );

        org.jdesktop.layout.GroupLayout layout = new org.jdesktop.layout.GroupLayout(getContentPane());
        getContentPane().setLayout(layout);
        layout.setHorizontalGroup(
            layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
            .add(layout.createSequentialGroup()
                .addContainerGap(356, Short.MAX_VALUE)
                .add(okButton, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, 67, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                .add(cancelButton)
                .addContainerGap())
            .add(layout.createSequentialGroup()
                .addContainerGap()
                .add(headerPanel, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                .addContainerGap(org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE))
        );

        layout.linkSize(new java.awt.Component[] {cancelButton, okButton}, org.jdesktop.layout.GroupLayout.HORIZONTAL);

        layout.setVerticalGroup(
            layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
            .add(org.jdesktop.layout.GroupLayout.TRAILING, layout.createSequentialGroup()
                .addContainerGap()
                .add(headerPanel, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, 292, Short.MAX_VALUE)
                .add(16, 16, 16)
                .add(layout.createParallelGroup(org.jdesktop.layout.GroupLayout.BASELINE)
                    .add(cancelButton)
                    .add(okButton))
                .addContainerGap())
        );
        pack();
    }// </editor-fold>//GEN-END:initComponents

    private void createDateTextFieldPropertyChange(java.beans.PropertyChangeEvent evt)//GEN-FIRST:event_createDateTextFieldPropertyChange
    {//GEN-HEADEREND:event_createDateTextFieldPropertyChange
        if(evt.getPropertyName().equals("value"))
        {
            this.date = (Date) this.createDateTextField.getValue();
            this.updateCurrencies();
        }
    }//GEN-LAST:event_createDateTextFieldPropertyChange

    private void typeComboBoxActionPerformed(java.awt.event.ActionEvent evt)//GEN-FIRST:event_typeComboBoxActionPerformed
    {//GEN-HEADEREND:event_typeComboBoxActionPerformed
        final LogicalFileType type =
            (LogicalFileType) this.typeComboBox.getSelectedItem();

        SwingUtilities.invokeLater(new Runnable()
        {
            public void run()
            {
                final boolean enabled = type != null && type.isSendByBank();
                bankDataTextField.setEnabled(enabled);
                searchBankDataButton.setEnabled(enabled);
            }
        });
    }//GEN-LAST:event_typeComboBoxActionPerformed

    private void okButtonActionPerformed(java.awt.event.ActionEvent evt)//GEN-FIRST:event_okButtonActionPerformed
    {
        if(this.getHeader() != null)
        {
            this.doClose(RET_OK);
        }
        else
        {
            UIManager.getLookAndFeel().provideErrorFeedback(
                getUserInterface().getParent());

        }
    }//GEN-LAST:event_okButtonActionPerformed

    private void cancelButtonActionPerformed(java.awt.event.ActionEvent evt)//GEN-FIRST:event_cancelButtonActionPerformed
    {
        this.doClose(RET_CANCEL);
    }//GEN-LAST:event_cancelButtonActionPerformed

    /** Closes the dialog */
    private void closeDialog(java.awt.event.WindowEvent evt)//GEN-FIRST:event_closeDialog
    {
        this.doClose(RET_CANCEL);
    }//GEN-LAST:event_closeDialog

    private void doClose(int retStatus)
    {
        this.returnStatus = retStatus;
        this.setVisible(false);
        this.dispose();
    }

    private void setForeground(final JLabel label, final Color color)
    {
        if(!this.colors.containsKey(label))
        {
            this.colors.put(label, label.getForeground());
        }
        label.setForeground(color);
    }

    private void resetForeground(final JLabel label)
    {
        if(this.colors.containsKey(label))
        {
            label.setForeground(this.colors.get(label));
        }
    }

    private void updateCurrencies()
    {
        final Currency[] currencies = getCurrencyDirectory().
            getDtausCurrencies(this.date != null ? this.date : new Date());

        Arrays.sort(currencies, new Comparator()
        {
            public int compare(final Object o1, final Object o2)
            {
                assert o1 instanceof Currency;
                assert o2 instanceof Currency;

                return ((Currency) o1).getCurrencyCode().
                    compareTo(((Currency) o2).getCurrencyCode());

            }
        });

        final Object prevSelection = currencyComboBox.getSelectedItem();
        currencyComboBox.removeAllItems();
        for(int i = 0; i < currencies.length; i++)
        {
            currencyComboBox.addItem(currencies[i]);
        }

        currencyComboBox.setSelectedItem(prevSelection);
    }

    private void updateFileTypes()
    {
        final List<LogicalFileType> types =
            new ArrayList<LogicalFileType>(50);

        if(isDebitSupported())
        {
            types.addAll(Arrays.asList(LogicalFileType.
                search(true, false, true, false)));

            types.addAll(Arrays.asList(LogicalFileType.
                search(true, false, false, true)));

            types.addAll(Arrays.asList(LogicalFileType.
                search(true, false, true, true)));

            if(isRemittanceSupported())
            {
                types.addAll(Arrays.asList(LogicalFileType.
                    search(true, true, true, false)));

                types.addAll(Arrays.asList(LogicalFileType.
                    search(true, true, false, true)));

                types.addAll(Arrays.asList(LogicalFileType.
                    search(true, true, true, true)));

            }
        }
        if(isRemittanceSupported())
        {
            types.addAll(Arrays.asList(LogicalFileType.
                search(false, true, true, false)));

            types.addAll(Arrays.asList(LogicalFileType.
                search(false, true, false, true)));

            types.addAll(Arrays.asList(LogicalFileType.
                search(false, true, true, true)));

            if(isDebitSupported())
            {
                types.addAll(Arrays.asList(LogicalFileType.
                    search(true, true, true, false)));

                types.addAll(Arrays.asList(LogicalFileType.
                    search(true, true, false, true)));

                types.addAll(Arrays.asList(LogicalFileType.
                    search(true, true, true, true)));

            }
        }

        final Object prevSelection = typeComboBox.getSelectedItem();
        typeComboBox.removeAllItems();
        for(final LogicalFileType type : types)
        {
            typeComboBox.addItem(type);
        }

        typeComboBox.setSelectedItem(prevSelection);
    }

    private boolean isBankcodeValid(final Bankleitzahl bankcode)
    {
        boolean valid = bankcode != null;

        if(valid)
        {
            valid = this.getBankleitzahlenVerzeichnis().
                getHeadOffice(bankcode) != null;

        }

        return valid;
    }

    // Variables declaration - do not modify//GEN-BEGIN:variables
    private javax.swing.JLabel accountLabel;
    private javax.swing.JTextField accountTextField;
    private javax.swing.JLabel bankDataLabel;
    private javax.swing.JTextField bankDataTextField;
    private javax.swing.JLabel bankLabel;
    private javax.swing.JTextField bankTextField;
    private javax.swing.JButton cancelButton;
    private javax.swing.JLabel createDateLabel;
    private javax.swing.JFormattedTextField createDateTextField;
    private javax.swing.JComboBox currencyComboBox;
    private javax.swing.JLabel currencyLabel;
    private javax.swing.JLabel customerLabel;
    private javax.swing.JTextField customerTextField;
    private javax.swing.JLabel executionDateLabel;
    private javax.swing.JFormattedTextField executionDateTextField;
    private javax.swing.JPanel headerPanel;
    private javax.swing.JButton okButton;
    private javax.swing.JLabel referenceLabel;
    private javax.swing.JTextField referenceTextField;
    private javax.swing.JButton searchBankDataButton;
    private javax.swing.JButton searchBankleitzahlButton;
    private javax.swing.JComboBox typeComboBox;
    private javax.swing.JLabel typeLabel;
    // End of variables declaration//GEN-END:variables

    private int returnStatus = RET_CANCEL;

    /**
     * Caches label colors.
     * @serial
     */
    private Map<JLabel, Color> colors = new HashMap(50);

    /**
     * Flag indicating if the dialog supports logical files which may hold
     * debit orders.
     * @serial
     */
    private boolean debitSupported;

    /**
     * Flag indicating if the dialog supports logical files which may hold
     * remittance orders.
     * @serial
     */
    private boolean remittanceSupported;


    /**
     * Date to be used for calculating the available currencies.
     * @serial
     */
    private Date date;

}
